import 'dart:convert';
import 'package:flutter/services.dart';

class AppConfig {
  final String environment;
  final String clientToken;
  final String clientSecretKey;
  final String clientCountry;
  final String clientDomain;
  final String clientPublishers;

  AppConfig(
      {this.environment,
      this.clientToken,
      this.clientSecretKey,
      this.clientCountry,
      this.clientDomain,
      this.clientPublishers});

  static Future<AppConfig> forEnvironment(String env) async {
    // set default to test if nothing was passed
    env = env ?? 'test';
    // load the json file
    final contents = await rootBundle.loadString(
      'lib/assets/config/$env.json',
    );

    // decode our json
    final json = jsonDecode(contents);

    // convert our JSON into an instance of our AppConfig class
    return AppConfig(
        environment: env,
        clientToken: json['com.dcoupon.sdk.ClientToken'],
        clientSecretKey: json['com.dcoupon.sdk.ClientSecretKey'],
        clientCountry: json['com.dcoupon.sdk.ClientCountry'],
        clientDomain: json['com.dcoupon.sdk.ClientDomain'],
        clientPublishers: jsonEncode(json['com.dcoupon.sdk.ClientPublishers']));
  }
}
