import 'dart:convert';

import 'package:dcoupon_sdk_flutter_package/src/domain/entities/client_publisher.dart';
import 'package:dcoupon_sdk_flutter_package/src/utils/constants.dart';

abstract class ConfigReader {
  static String get languageCode => _languageCode;
  static String _languageCode;

  static String get userJwt => _userJwt;
  static String _userJwt;

  static ClientType get clientType => _clientType;
  static ClientType _clientType;

  static String get clientDomain => _clientDomain;
  static String _clientDomain;

  static String get clientCountry => _clientCountry;
  static String _clientCountry;

  static String get clientApiToken => _clientApiToken;
  static String _clientApiToken;

  static String get clientSecretKey => _clientSecretKey;
  static String _clientSecretKey;

  static int get dateOffset => _dateOffset;
  static int _dateOffset = 0;

  static List<ClientPublisher> _clientPublishers;
  static String getPublisherTokenByName(
    String publisherName,
  ) {
    var publisherApiToken;
    _clientPublishers.forEach((element) {
      if (publisherName == null || publisherName.isEmpty) {
        // get the last item
        publisherApiToken = publisherApiToken = element.token;
      } else {
        // get searched item
        if (element.name == publisherName) {
          publisherApiToken = publisherApiToken = element.token;
        }
      }
    });
    if (publisherApiToken == null || publisherApiToken.isEmpty)
      throw Exception('miss-configured publishers not found');

    return publisherApiToken;
  }

  static void init(String contents) {
    try {
      if (contents == null || contents.isEmpty) {
        throw new Exception('contents can not be null');
      }
      // decode the json
      final json = jsonDecode(contents);

      _clientDomain = json['com.dcoupon.sdk.ClientDomain'];
      _clientCountry = json['com.dcoupon.sdk.ClientCountry'];
      _clientApiToken = json['com.dcoupon.sdk.ClientToken'];
      _clientSecretKey = json['com.dcoupon.sdk.ClientSecretKey'];

      final clientPublishersJson = json['com.dcoupon.sdk.ClientPublishers'];
      var clientPublishersList = <ClientPublisher>[];

      clientPublishersJson.forEach(
        (v) {
          clientPublishersList.add(
            ClientPublisher.fromJson(v),
          );
        },
      );

      _clientPublishers = List.from(clientPublishersList);
    } catch (ex) {
      print('loadAsJsonString: ' + ex.toString());
      return null;
    }
  }

  static void setLanguageCode(String languageCode) {
    _languageCode = languageCode;
  }

  static void setUserJwt(String jwt) {
    _userJwt = jwt;
  }

  static void setClientType(ClientType clientType) {
    _clientType = clientType;
  }

  static void setDateOffset(int dateOffset) {
    _dateOffset = dateOffset;
  }

  /// Returns if the app is configured properly
  static bool isSetUp() {
    var message = new List<String>();
    if (_clientApiToken == null || _clientApiToken.isEmpty) {
      message.add("com.dcoupon.sdk.ClientToken");
    }
    if (_clientSecretKey == null || _clientSecretKey.isEmpty) {
      message.add("com.dcoupon.sdk.ClientSecretKey");
    }
    if (_clientCountry == null || _clientCountry.isEmpty) {
      message.add("com.dcoupon.sdk.ClientCountry");
    }
    if (_clientDomain == null || _clientDomain.isEmpty) {
      message.add("com.dcoupon.sdk.ClientDomain");
    } else {
      if (!(_clientDomain == "local" ||
          _clientDomain == "test" ||
          _clientDomain == "dev" ||
          _clientDomain == "dcoupon.eu" ||
          _clientDomain == "dcoupon.com"))
        message.add("com.dcoupon.sdk.ClientDomain not allowed");
    }
    if (_clientPublishers.isEmpty) {
      message.add("com.dcoupon.sdk.ClientPublishers");
    }
    if (message.isNotEmpty) {
      print("Missed manadatory parameter/s:");
      message.forEach((str) {
        print(str);
      });
      return false;
    } else {
      return true;
    }
  }
}
