import 'package:dartz/dartz.dart';
import 'package:dcoupon_sdk_flutter_package/src/config_reader.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/exceptions.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/failures.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/helpers/publisher_api_helper.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/store_get_stores_query.dart';
import 'package:dcoupon_sdk_flutter_package/src/utils/constants.dart';

import 'package:dio/dio.dart';

abstract class StoreRemoteDataSource {
  Future<Either<Failure, dynamic>> getStores({
    String promotionToken,
    int start,
    int limit,
    List<String> retailerTokens,
    String zipcode,
    String latitude,
    String longitude,
    String radius,
    String publisherName,
  });
}

class StoreRemoteDataSourceImpl implements StoreRemoteDataSource {
  Dio dio = Dio();

  @override
  Future<Either<Failure, dynamic>> getStores({
    String promotionToken,
    int start,
    int limit,
    List<String> retailerTokens,
    String zipcode,
    String latitude,
    String longitude,
    String radius,
    String publisherName,
  }) async {
    dio.options.headers = PublisherApiHelper.buildHeaders();

    final publisherId = ConfigReader.getPublisherTokenByName(publisherName);
    final storeGetStoresQuery = StoreGetStoresQuery(
      lang: ConfigReader.languageCode,
      country: ConfigReader.clientCountry,
      promotionToken: promotionToken,
      start: start,
      limit: limit,
      retailerTokens: ConfigReader.clientType != ClientType.RETAILER
          ? retailerTokens
          : [ConfigReader.clientApiToken],
      zipcode: zipcode,
      latitude: latitude,
      longitude: longitude,
      radius: radius,
      publisherId: publisherId,
    );

    try {
      final response = await dio.get(
        PublisherApiHelper.getStoresUrl(),
        queryParameters: storeGetStoresQuery.toJson(),
      );
      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.RESPONSE) {
        return Left(ServerFailure(httpStatus: e.response.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }
}
