import 'package:dcoupon_sdk_flutter_package/src/core/error/exceptions.dart';
import 'package:dcoupon_sdk_flutter_package/src/utils/constants.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:meta/meta.dart';

abstract class UserLocalDataSource {
  Future<void> cacheJwt(String jwt);

  Future<String> getJwt();

  Future<void> cacheClientType(String clientType);

  Future<String> getClientType();

  Future<void> clearSharedPreferences();
}

class UserLocalDataSourceImpl implements UserLocalDataSource {
  final SharedPreferences sharedPreferences;

  UserLocalDataSourceImpl({
    @required this.sharedPreferences,
  });

  @override
  Future<void> cacheJwt(String jwt) {
    return sharedPreferences.setString(
      Constants.JWT,
      jwt,
    );
  }

  @override
  Future<String> getJwt() {
    try {
      final jwt = sharedPreferences.getString(
        Constants.JWT,
      );
      return Future.value(jwt);
    } catch (_) {
      throw CacheException();
    }
  }

  @override
  Future<void> cacheClientType(String clientType) {
    return sharedPreferences.setString(
      Constants.CLIENT_TYPE,
      clientType,
    );
  }

  @override
  Future<String> getClientType() {
    try {
      final clientType = sharedPreferences.getString(
        Constants.CLIENT_TYPE,
      );
      return Future.value(clientType);
    } catch (_) {
      throw CacheException();
    }
  }

  @override
  Future<void> clearSharedPreferences() async {
    try {
      return await sharedPreferences.clear();
    } catch (_) {
      throw CacheException();
    }
  }
}
