import 'dart:convert';

import 'package:dio/dio.dart';
import 'package:flutter/foundation.dart';

class LoggingInterceptor extends Interceptor {
  @override
  Future onError(DioError err) async {
    debugPrint("""ERROR:
    URL: ${err.request.uri}\n
    Method: ${err.request.method} 
    Headers: ${jsonEncode(err.response.headers.map)}
    StatusCode: ${err.response.statusCode}
    Data: ${jsonEncode(err.response.data)}
    <-- END HTTP
        """, wrapWidth: 1024);
    return super.onError(err);
  }

  @override
  Future onRequest(RequestOptions options) async {
    debugPrint("""REQUEST:
    ${cURLRepresentation(options)}
    """, wrapWidth: 1024);
    return super.onRequest(options);
  }

  @override
  Future onResponse(Response response) async {
    debugPrint("""RESPONSE:
    URL: ${response.request.uri}
    Method: ${response.request.method}
    Headers: ${jsonEncode(response.request.headers)}
    Data: ${jsonEncode(response.data)}
        """, wrapWidth: 1024);
    return super.onResponse(response);
  }

  String cURLRepresentation(RequestOptions options) {
    List<String> components = ["\$ curl -i"];
    if (options.method != null && options.method.toUpperCase() == "GET") {
      components.add("-X ${options.method}");
    }

    if (options.headers != null) {
      options.headers.forEach((k, v) {
        if (k != "Cookie") {
          components.add("-H \"$k: $v\"");
        }
      });
    }

    var data = jsonEncode(options.data);
    if (data != null) {
      data = data.replaceAll('\"', '\\\"');
      components.add("-d \"$data\"");
    }

    components.add("\"${options.uri.toString()}\"");

    return components.join('\\\n\t');
  }
}
