import 'package:dcoupon_sdk_flutter_package/src/config_reader.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/helpers/authorization_helper.dart';
import 'package:dcoupon_sdk_flutter_package/src/utils/constants.dart';
import 'package:dcoupon_sdk_flutter_package/src/utils/enum_helper.dart';

const String VERSION = "v1";

const Map<String, String> URLS_BY_ENVIRONMENT = {
  'local': 'http://localhost:3000/',
  'test': 'https://testapi.dcoupon.com/thirdparty/',
  'dev': 'https://services-dev.dcoupon.com/thirdparty/',
  'dcoupon.eu': 'https://services.dcoupon.eu/thirdparty/',
  'dcoupon.com': 'https://services.dcoupon.com/thirdparty/'
};

class ThirdPartyAPIHelper {
  static Map<String, dynamic> buildHeaders({
    AuthMethod authMethod = AuthMethod.SIGNATURE,
  }) {
    var headers = <String, dynamic>{};

    final currentDateFormatted = AuthorizationHelper.getCurrentDateFormatted();

    headers[Constants.HEADER_AUTH_METHOD] = EnumHelper.enumToString(authMethod);
    headers[Constants.HEADER_AUTH_TIMESTAMP] = currentDateFormatted;
    headers[Constants.HEADER_AUTH_APIKEY] = ConfigReader.clientApiToken;

    switch (authMethod) {
      case AuthMethod.SIGNATURE:
        headers[Constants.HEADER_AUTH_SIGNATURE] =
            AuthorizationHelper.getAuthSignature(currentDateFormatted);
        break;
      case AuthMethod.JWT:
        headers[Constants.HEADER_AUTH_SIGNATURE] = ConfigReader.userJwt;
        break;
    }

    return headers;
  }

  static String userLoginUrl() {
    return URLS_BY_ENVIRONMENT[ConfigReader.clientDomain] +
        "login-sdk/" +
        VERSION;
  }

  static String getEnrollmentFormUrl() {
    return URLS_BY_ENVIRONMENT[ConfigReader.clientDomain] +
        "enrollmentform-sdk/" +
        VERSION;
  }

  static String acceptEnrollmentFormUrl() {
    return URLS_BY_ENVIRONMENT[ConfigReader.clientDomain] +
        "acceptenrollment-sdk/" +
        VERSION;
  }
}
