import 'package:dartz/dartz.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/failures.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_accept_enrollment_form_response.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_get_enrollment_form_response.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_login_response.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/entities/legal_document.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/entities/user.dart';
import 'package:meta/meta.dart';

abstract class UserRepository {
  Future<Either<Failure, UserLoginResponse>> logIn({
    @required String externalId,
    @required User user,
  });

  Future<Either<Failure, bool>> isLoggedIn();

  Future<Either<Failure, void>> logOut();

  Future<Either<Failure, UserGetEnrollmentFormResponse>> getEnrollmentForm();

  Future<Either<Failure, UserAcceptEnrollmentFormResponse>>
      acceptEnrollmentForm({
    @required String externalId,
    @required User user,
    @required List<LegalDocument> legalDocumentsAccepted,
  });

  Future<Either<Failure, dynamic>> getUserData();

  Future<Either<Failure, dynamic>> updateUserData({
    String alias,
    String gender,
    String birthdate,
  });
}
