import 'package:dartz/dartz.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/failures.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/usecases/usecase.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/repositories/coupon_repository.dart';
import 'package:equatable/equatable.dart';

class CouponCreateUserCoupons
    implements UseCase<dynamic, CouponCreateUserCouponsParams> {
  final CouponRepository repository;

  CouponCreateUserCoupons(this.repository);

  @override
  Future<Either<Failure, dynamic>> call(
      CouponCreateUserCouponsParams params) async {
    return await repository.createUserCoupons(
      promotionTokens: params.promotionTokens,
      latitude: params.latitude,
      longitude: params.longitude,
      source: params.source,
      transId: params.transId,
      crmId: params.crmId,
      publisherName: params.publisherName,
    );
  }
}

class CouponCreateUserCouponsParams extends Equatable {
  final List<String> promotionTokens;
  final String latitude;
  final String longitude;
  final String source;
  final String transId;
  final String crmId;
  final String publisherName;

  CouponCreateUserCouponsParams({
    this.promotionTokens,
    this.latitude,
    this.longitude,
    this.source,
    this.transId,
    this.crmId,
    this.publisherName,
  });

  @override
  List<Object> get props => [
        promotionTokens,
        latitude,
        longitude,
        source,
        transId,
        crmId,
        publisherName,
      ];
}
