import 'package:dartz/dartz.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/failures.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/usecases/usecase.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/repositories/loyalty_repository.dart';
import 'package:equatable/equatable.dart';

class LoyaltyAddOrUpdateLoyaltyCard
    implements UseCase<dynamic, LoyaltyAddOrUpdateLoyaltyCardParams> {
  final LoyaltyRepository repository;

  LoyaltyAddOrUpdateLoyaltyCard(this.repository);

  @override
  Future<Either<Failure, dynamic>> call(
      LoyaltyAddOrUpdateLoyaltyCardParams params) async {
    return await repository.addOrUpdateLoyaltyCard(
      affiliateApiToken: params.affiliateApiToken,
      cardNumber: params.cardNumber,
    );
  }
}

class LoyaltyAddOrUpdateLoyaltyCardParams extends Equatable {
  final String affiliateApiToken;
  final String cardNumber;

  LoyaltyAddOrUpdateLoyaltyCardParams({
    this.affiliateApiToken,
    this.cardNumber,
  });

  @override
  List<Object> get props => [
        affiliateApiToken,
        cardNumber,
      ];
}
