import 'package:dartz/dartz.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/failures.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/usecases/usecase.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/legal_document_model.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_accept_enrollment_form_response.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/entities/user.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/repositories/user_repository.dart';
import 'package:meta/meta.dart';
import 'package:equatable/equatable.dart';

class UserAcceptEnrollmentForm
    implements
        UseCase<UserAcceptEnrollmentFormResponse,
            UserAcceptEnrollmentFormParams> {
  final UserRepository repository;

  UserAcceptEnrollmentForm(this.repository);

  @override
  Future<Either<Failure, UserAcceptEnrollmentFormResponse>> call(
      UserAcceptEnrollmentFormParams params) async {
    return await repository.acceptEnrollmentForm(
      externalId: params.externalId,
      user: params.user,
      legalDocumentsAccepted: params.legalDocumentsAccepted,
    );
  }
}

class UserAcceptEnrollmentFormParams extends Equatable {
  final String externalId;
  final User user;
  final List<LegalDocumentModel> legalDocumentsAccepted;

  UserAcceptEnrollmentFormParams({
    @required this.externalId,
    @required this.user,
    @required this.legalDocumentsAccepted,
  });

  @override
  List<Object> get props => [
        externalId,
        user,
        legalDocumentsAccepted,
      ];
}
