import 'package:dartz/dartz.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/failures.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/usecases/usecase.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_login_response.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/entities/user.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/repositories/user_repository.dart';
import 'package:meta/meta.dart';
import 'package:equatable/equatable.dart';

class UserLogIn implements UseCase<UserLoginResponse, UserLogInParams> {
  final UserRepository repository;

  UserLogIn(this.repository);

  @override
  Future<Either<Failure, UserLoginResponse>> call(
      UserLogInParams params) async {
    final user = User(
      email: params.email,
      alias: params.alias,
      gender: params.gender,
      birthdate: params.birthdate,
    );
    return await repository.logIn(
      externalId: params.externalId,
      user: user,
    );
  }
}

class UserLogInParams extends Equatable {
  final String email;
  final String externalId;
  final String alias;
  final String gender;
  final String birthdate;

  UserLogInParams({
    @required this.email,
    @required this.externalId,
    this.alias,
    this.gender,
    this.birthdate,
  });

  @override
  List<Object> get props => [
        email,
        externalId,
        alias,
        gender,
        birthdate,
      ];
}
