import 'dart:io';

import 'package:dcoupon_sdk_flutter_package/src/domain/entities/enrollment_form.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/entities/user.dart';
import 'package:dcoupon_sdk_flutter_package/src/presentation/colors.dart';
import 'package:dcoupon_sdk_flutter_package/src/presentation/widgets/bottom_picker_widget.dart';
import 'package:dcoupon_sdk_flutter_package/src/utils/constants.dart';
import 'package:dcoupon_sdk_flutter_package/src/utils/translations.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_html/flutter_html.dart';
import 'package:flutter_html/style.dart';
import 'package:intl/intl.dart';

class EnrollmentFormScreen extends StatefulWidget {
  final EnrollmentForm enrollmentForm;
  final VoidCallback onCloseEnrollment;
  final Function(User updatedUser) onAcceptEnrollment;

  EnrollmentFormScreen({
    Key key,
    this.enrollmentForm,
    this.onCloseEnrollment,
    this.onAcceptEnrollment,
  }) : super(key: key);

  @override
  _EnrollmentFormScreenState createState() => _EnrollmentFormScreenState();
}

class _EnrollmentFormScreenState extends State<EnrollmentFormScreen> {
  var _checkboxIsSelected = false;
  var _isVisibleTextLegalInfo = true;
  var _isEnableNextBtn = false;
  var _isLoading = false;
  var _formKey = GlobalKey<FormState>();

  var legalTextPP, legalTextTOU;
  String _currentAlias;
  String _currentGender;
  DateTime selectedDate;
  List<dynamic> legalDocumentsToAccept = new List<dynamic>();
  DateFormat dateFormat;
  DateFormat dateFormatToSend = new DateFormat("yyyy-MM-dd");
  DateTime _maxDateAllowed = new DateTime(
      DateTime.now().year - 16, DateTime.now().month, DateTime.now().day);
  ScrollController _scrollController;
  bool _acceptRequest = false;

  @override
  void initState() {
    super.initState();
    _scrollController = ScrollController();

    _currentAlias = widget.enrollmentForm.user.alias;
    _currentGender = _validateGender(widget.enrollmentForm.user.gender)
        ? widget.enrollmentForm.user.gender
        : Constants.RATHER_NOT_SAY;
    final birthdate = _validateBirthdate(widget.enrollmentForm.user.birthdate)
        ? widget.enrollmentForm.user.birthdate
        : null;
    selectedDate = birthdate != null ? DateTime.parse(birthdate) : null;

    legalTextPP = widget.enrollmentForm.legalDocumentsList
        .firstWhere((element) => element.docType == Constants.PP)
        .legalText;
    legalTextTOU = widget.enrollmentForm.legalDocumentsList
        .firstWhere((element) => element.docType == Constants.TOU)
        .legalText;
  }

  void didChangeDependencies() {
    super.didChangeDependencies();
    dateFormat =
        new DateFormat.yMd(Localizations.localeOf(context).languageCode);
  }

  @override
  void dispose() {
    if (!_acceptRequest) {
      closeEnrollment();
    }
    super.dispose();
  }

  void closeEnrollment() {
    widget.onCloseEnrollment();
  }

  @override
  Widget build(BuildContext context) {
    return DefaultTabController(
      length: 3,
      child: Scaffold(
        appBar: AppBar(
            // iconTheme: IconThemeData(color: Colors.black),
            // brightness: Brightness.dark,
            // backgroundColor: Colors.white,
            bottom: TabBar(
              isScrollable: true,
              // labelColor: Colors.black,
              // indicatorColor: Colors.orange,
              tabs: [
                Tab(text: Translations.of(context).text('dcoupon_welcome')),
                Tab(text: Translations.of(context).text('dcoupon_legal_pp')),
                Tab(text: Translations.of(context).text('dcoupon_legal_tou')),
              ],
            ),
            centerTitle: true,
            title: new Image(
              fit: BoxFit.scaleDown,
              height: 40,
              image: new AssetImage('lib/assets/images/dcoupon_logo.png',
                  package: 'dcoupon_sdk_flutter_package'),
            )),
        body: TabBarView(
          children: [
            Container(
              child: SingleChildScrollView(
                controller: _scrollController,
                child: Visibility(
                  visible: _isVisibleTextLegalInfo,
                  child: Container(
                    padding: EdgeInsets.all(8.0),
                    child: new Column(
                      children: <Widget>[
                        new Row(
                          crossAxisAlignment: CrossAxisAlignment.center,
                          children: [
                            Checkbox(
                                value: _checkboxIsSelected,
                                onChanged: (bool value) {
                                  setState(() {
                                    _checkboxIsSelected = value;
                                    _isEnableNextBtn =
                                        _checkboxIsSelected ? true : false;
                                    if (_isEnableNextBtn) {
                                      _scrollBottom();
                                    }
                                  });
                                }),
                            Expanded(
                                child: Text(Translations.of(context)
                                    .text('dcoupon_check_box_text'))),
                          ],
                        ),
                        Container(
                            height: MediaQuery.of(context).size.height * 0.6,
                            child: SingleChildScrollView(
                                child: Html(
                              data: Translations.of(context)
                                  .text('dcoupon_legal_info_text'),
                              //Optional parameters:
                              style: {
                                "a": Style(
                                  color: Colors.blue,
                                  textDecorationColor: Colors.blueAccent,
                                  textDecoration: TextDecoration.underline,
                                ),
                              },
                            ))),
                        new Column(
                            mainAxisSize: MainAxisSize.max,
                            mainAxisAlignment: MainAxisAlignment.end,
                            children: <Widget>[
                              Align(
                                alignment: Alignment.bottomRight,
                                child: RaisedButton.icon(
                                  color: DcouponColors.dCouponPrimaryOrange,
                                  icon: Icon(
                                    Icons.arrow_forward,
                                    color: Colors.white,
                                  ),
                                  onPressed: (_isEnableNextBtn)
                                      ? () => isEnableNextBtn()
                                      : null,
                                  label: Text(
                                    Translations.of(context)
                                        .text('dcoupon_next'),
                                    style: TextStyle(
                                        fontSize: 16, color: Colors.white),
                                  ),
                                ),
                              ),
                            ]),
                      ],
                    ),
                  ),
                  replacement: Stack(
                    children: <Widget>[
                      Container(
                        padding: EdgeInsets.all(8.0),
                        child: Column(
                          children: <Widget>[
                            Form(
                                key: _formKey,
                                child: Column(
                                  crossAxisAlignment: CrossAxisAlignment.start,
                                  children: <Widget>[
                                    SizedBox(height: 24),
                                    TextFormField(
                                      readOnly: true,
                                      decoration: InputDecoration(
                                        labelText:
                                            Translations.of(context).text(
                                          'dcoupon_email',
                                        ),
                                        border: OutlineInputBorder(),
                                      ),
                                      initialValue:
                                          widget.enrollmentForm.user.email,
                                      validator: (value) {
                                        if (value.isEmpty) {
                                          return (Translations.of(context).text(
                                                  'dcoupon_missed_parameter') +
                                              " " +
                                              Translations.of(context)
                                                  .text('dcoupon_email'));
                                        }
                                        return null;
                                      },
                                    ),
                                    SizedBox(height: 16),
                                    TextFormField(
                                      decoration: InputDecoration(
                                        labelText: Translations.of(context)
                                            .text('dcoupon_alias'),
                                        border: OutlineInputBorder(),
                                      ),
                                      initialValue:
                                          widget.enrollmentForm.user.alias,
                                      onChanged: (value) {
                                        if (value.length > 0)
                                          setState(() {
                                            _currentAlias = value;
                                          });
                                      },
                                      validator: (value) {
                                        if (value.isEmpty) {
                                          return (Translations.of(context).text(
                                                  'dcoupon_missed_parameter') +
                                              " " +
                                              Translations.of(context)
                                                  .text('dcoupon_alias'));
                                        }
                                        return null;
                                      },
                                    ),
                                    SizedBox(height: 16),
                                    InputDecorator(
                                      decoration: InputDecoration(
                                        contentPadding:
                                            EdgeInsets.fromLTRB(12, 12, 8, 0),
                                        labelText: Translations.of(context)
                                            .text('dcoupon_gender'),
                                        border: OutlineInputBorder(),
                                      ),
                                      child: DropdownButton(
                                        underline: Container(),
                                        isExpanded: true,
                                        hint: new Text(Translations.of(context)
                                                .text(
                                                    'dcoupon_missed_parameter') +
                                            " " +
                                            Translations.of(context)
                                                .text('dcoupon_gender')),
                                        value: _currentGender,
                                        items: [
                                          DropdownMenuItem(
                                            value: Constants.MALE,
                                            child: Text(
                                              Translations.of(context)
                                                  .text('dcoupon_gender_male'),
                                            ),
                                          ),
                                          DropdownMenuItem(
                                            value: Constants.FEMALE,
                                            child: Text(
                                              Translations.of(context).text(
                                                  'dcoupon_gender_female'),
                                            ),
                                          ),
                                          DropdownMenuItem(
                                            value: Constants.OTHER,
                                            child: Text(
                                              Translations.of(context)
                                                  .text('dcoupon_gender_other'),
                                            ),
                                          ),
                                          DropdownMenuItem(
                                            value: Constants.RATHER_NOT_SAY,
                                            child: Text(
                                              Translations.of(context).text(
                                                  'dcoupon_gender_rather_not_say'),
                                            ),
                                          ),
                                        ],
                                        onChanged: changedDropDownItemGender,
                                      ),
                                    ),
                                    SizedBox(height: 16),
                                    TextFormField(
                                      controller: TextEditingController()
                                        ..text = selectedDate != null
                                            ? dateFormat.format(selectedDate)
                                            : null,
                                      readOnly: true,
                                      autofocus: false,
                                      decoration: InputDecoration(
                                        labelText: Translations.of(context)
                                            .text('dcoupon_birthdate'),
                                        border: OutlineInputBorder(),
                                      ),
                                      onTap: () => _selectDate(context),
                                    ),
                                    SizedBox(height: 16),
                                    new Row(
                                        mainAxisSize: MainAxisSize.max,
                                        mainAxisAlignment:
                                            MainAxisAlignment.spaceBetween,
                                        crossAxisAlignment:
                                            CrossAxisAlignment.center,
                                        children: <Widget>[
                                          RaisedButton(
                                              color: Colors.grey,
                                              onPressed: () {
                                                setState(() {
                                                  _isVisibleTextLegalInfo =
                                                      true;
                                                });
                                                // Validate will return true if the form is valid, or false if
                                                // the form is invalid.
                                                if (_formKey.currentState
                                                    .validate()) {
                                                  // Process data.
                                                }
                                              },
                                              child: Row(
                                                crossAxisAlignment:
                                                    CrossAxisAlignment.center,
                                                mainAxisAlignment:
                                                    MainAxisAlignment.start,
                                                mainAxisSize: MainAxisSize.min,
                                                children: <Widget>[
                                                  Icon(Icons.arrow_back,
                                                      color: Colors.white),
                                                  Text(
                                                      Translations.of(context)
                                                          .text('dcoupon_back'),
                                                      style: TextStyle(
                                                          fontSize: 16,
                                                          color: Colors.white)),
                                                ],
                                              )),
                                          RaisedButton(
                                              color: DcouponColors
                                                  .dCouponPrimaryOrange,
                                              onPressed: _isLoading
                                                  ? null
                                                  : () {
                                                      setState(() {});
                                                      // Validate will return true if the form is valid, or false if
                                                      // the form is invalid.
                                                      if (_formKey.currentState
                                                          .validate()) {
                                                        // Process data.
                                                        acceptEnrollment();
                                                      }
                                                    },
                                              child: Row(
                                                crossAxisAlignment:
                                                    CrossAxisAlignment.center,
                                                mainAxisAlignment:
                                                    MainAxisAlignment.start,
                                                mainAxisSize: MainAxisSize.min,
                                                children: <Widget>[
                                                  Text(
                                                      Translations.of(context)
                                                          .text(
                                                              'dcoupon_accept'),
                                                      style: TextStyle(
                                                          fontSize: 16,
                                                          color: Colors.white)),
                                                  Icon(Icons.check,
                                                      color: Colors.white)
                                                ],
                                              ))
                                        ]),
                                  ],
                                )),
                          ],
                        ),
                      ),
                      Container(
                        child: _isLoading
                            ? Container(
                                color: Colors.grey[200].withAlpha(150),
                                height: MediaQuery.of(context).size.height,
                                child: Center(
                                  child: CircularProgressIndicator(),
                                ),
                              )
                            : Container(),
                      ),
                    ],
                  ),
                ),
              ),
            ),
            new Container(
              child: SingleChildScrollView(
                padding: EdgeInsets.all(8.0),
                child: Html(
                  data: legalTextPP,
                  style: {
                    "a": Style(
                      color: Colors.blue,
                      textDecorationColor: Colors.blueAccent,
                      textDecoration: TextDecoration.underline,
                    ),
                  },
                ),
              ),
            ),
            new Container(
              child: SingleChildScrollView(
                padding: EdgeInsets.all(8.0),
                child: Html(
                  data: legalTextTOU,
                  style: {
                    "a": Style(
                      color: Colors.blue,
                      textDecorationColor: Colors.blueAccent,
                      textDecoration: TextDecoration.underline,
                    ),
                  },
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }

  void isEnableNextBtn() {
    setState(() {
      _isVisibleTextLegalInfo = false;
    });
  }

  void acceptEnrollment() {
    setState(() {
      _isLoading = true;
      _acceptRequest = true;
    });

    final birthdateString =
        selectedDate != null ? dateFormatToSend.format(selectedDate) : '';

    final updatedUser = User(
      email: widget.enrollmentForm.user.email,
      alias: _currentAlias,
      gender: _currentGender,
      birthdate: birthdateString,
    );

    widget.onAcceptEnrollment(updatedUser);
  }

  Future<Null> _selectDate(BuildContext context) async {
    if (Platform.isIOS) {
      DateTime picked = selectedDate ?? _maxDateAllowed;
      await showCupertinoModalPopup(
        context: context,
        builder: (BuildContext context) => BottomPickerWidget(
          cupertinoPicker: CupertinoDatePicker(
            mode: CupertinoDatePickerMode.date,
            onDateTimeChanged: (DateTime date) {
              picked = date;
            },
            initialDateTime: picked,
          ),
        ),
      );
      _updateSelectedDate(picked);
    } else {
      final DateTime picked = await showDatePicker(
          context: context,
          initialDatePickerMode: DatePickerMode.year,
          initialDate: selectedDate ?? _maxDateAllowed,
          firstDate: DateTime(1900),
          lastDate: _maxDateAllowed);
      _updateSelectedDate(picked);
    }
  }

  void _updateSelectedDate(DateTime picked) {
    if (picked != null &&
        picked != selectedDate &&
        picked.millisecondsSinceEpoch <
            _maxDateAllowed.millisecondsSinceEpoch) {
      setState(() {
        selectedDate = picked;
      });
    }
  }

  void changedDropDownItemGender(String selectedGender) {
    setState(() {
      _currentGender = selectedGender;
    });
  }

  bool _validateGender(String gender) {
    switch (gender) {
      case Constants.MALE:
      case Constants.FEMALE:
      case Constants.OTHER:
      case Constants.RATHER_NOT_SAY:
        return true;
        break;
      default:
        return false;
        break;
    }
  }

  bool _validateBirthdate(String date) {
    try {
      if (date != null && date.isNotEmpty && date != 'None') {
        return _maxDateAllowed.isAfter(DateTime.parse(date)) ? true : false;
      }
      return false;
    } catch (ex) {
      print(ex.toString());
      return false;
    }
  }

  void _scrollBottom() {
    _scrollController.animateTo(_scrollController.position.maxScrollExtent,
        duration: Duration(milliseconds: 200), curve: Curves.linear);
  }
}
